<?php

App::uses('AppComponent', 'Controller/Component');
App::uses('CakeText', 'Utility');
App::uses('InflectorExt', 'EvInflector.Lib');

class MetaComponent extends AppComponent {

/**
 * Default settings for this Component
 *
 * @var array
 */
	protected $_defaultSettings = [
		'descriptionField' => 'content', // The field from the model to use to set the meta description
		'descriptionLength' => 200 // The length of the meta description when using model fields
	];

	/**
	 * Calls parent. Merges default settings with any settings passed to the controller
	 *
	 * @see Component::initialize()
	 */
	public function initialize(Controller $controller) {
		parent::initialize($controller);

		$this->settings = array_merge($this->_defaultSettings, $this->settings);
	}

	/**
	 * Sets the view variables for the meta data.
	 *
	 * @param array $data array containing the MetaData
	 * @param string $overrideModel The main model, useful if setting Page content from within a plugin
	 */
	public function set($data, $overrideModel = null) {
		$Model = $this->_controller->{$this->_controller->modelClass};

		// override the Model, otherwise plugins will look at their own models
		// for the title when it may actually want to look at 'Page' Model.
		if (! empty($overrideModel)) {
			$this->_controller->loadModel($overrideModel);
			list($pluginName, $modelName) = pluginSplit($overrideModel);
			$Model = $this->_controller->{$modelName};
		}

		// Set the title for layout, either using the meta title, display
		// field, or model name.
		$title = $this->_getTitleForLayout($data, $Model);

		$description = $this->_getDescriptionForLayout($data, $Model);

		$keywords = $this->_getKeywordsForLayout($data, $Model);

		$this->_controller->set('titleForLayout', $title);

		// Set title_for_layout for backwards compatibility (CakePhp 2.4.x).
		$this->_controller->set('title_for_layout', $title);

		$this->_controller->set('metaTitle', $title);
		$this->_controller->set('metaDescription', $description);
		$this->_controller->set('metaKeywords', $keywords);

		return;
	}

	/**
	 * Set the title for layout, either using the meta title, display field, or model name.
	 * @param  array  $data  The data passed through from the controller
	 * @param  object $Model The model to check available fields
	 * @return string        The meta title to set
	 */
	protected function _getTitleForLayout($data, $Model) {
		if (!empty($data['MetaData']['title'])) {
			$titleForLayout = $data['MetaData']['title'];
		} elseif (isset($data[$Model->alias][$Model->displayField])) {
			$titleForLayout = $data[$Model->alias][$Model->displayField] . ' | ' . Configure::read('SiteSetting.general.site_title');
		} else {
			$titleForLayout = InflectorExt::pluralize($Model->displayName) . ' | ' . Configure::read('SiteSetting.general.site_title');
		}

		return $titleForLayout;
	}

	/**
	 * Set the description for layout, either using the meta description, overriden description field, body or description field.
	 * @param  array  $data  The data passed through from the controller
	 * @param  object $Model The model to check available fields
	 * @return string        The meta description to set
	 */
	protected function _getDescriptionForLayout($data, $Model) {
		if (!empty($data['MetaData']['description'])) {
			return $data['MetaData']['description'];
		}

		$defaultFields = [
			'body',
			'description'
		];
		$fields = array_merge((array)$this->settings['descriptionField'], $defaultFields);
		foreach ($fields as $field) {
			if (isset($data[$Model->alias][$field])) {

				// Strip the tags first as they're not needed
				$string = strip_tags($data[$Model->alias][$field]);

				// Trim the content so the regex doesn't crash apache. This was an issue with MEDIUMTEXT
				// fields in the db as trying to regex a huge string caused apache to segfault.
				$string = substr($string, 0, $this->settings['descriptionLength'] * 4);

				// Replace new lines and replace whitespace with a single space
				$string = preg_replace(
					[
						"/\r|\n/",
						"/\s+/"
					],
					" ",
					$string
				);

				// Finally make it the correct length with ellipsis if required
				return CakeText::truncate($string, $this->settings['descriptionLength']);
			}
		}

		return null;
	}

	/**
	 * Set the description for layout, using the meta keywords or null.
	 * @param  array  $data  The data passed through from the controller
	 * @param  object $Model The model to check available fields
	 * @return string        The meta keywords to set
	 */
	protected function _getKeywordsForLayout($data, $Model) {
		if (!empty($data['MetaData']['keywords'])) {
			$keywordsForLayout = $data['MetaData']['keywords'];
		} else {
			$keywordsForLayout = null;
		}

		return $keywordsForLayout;
	}
}
